/*
 * Copyright (c) 2019 Nordic Semiconductor
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#define DT_DRV_COMPAT adi_adxl355

#ifndef __CUSTOM_ADXL355_DRIVER_H__
#define __CUSTOM_ADXL355_DRIVER_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <zephyr/device.h>

typedef	void (*custom_adxl355_api_print_t)(const struct device * dev);
typedef	int  (*custom_adxl355_api_open_t)(const struct device * dev);
typedef int  (*custom_adxl355_api_read_reg_t)(const struct device * dev, uint8_t reg, uint8_t *data, int size);
typedef int  (*custom_adxl355_api_write_reg_t)(const struct device * dev, uint8_t reg, uint8_t value);
typedef int  (*custom_adxl355_api_close_t)(const struct device * dev);


struct custom_adxl355_driver_api {
	custom_adxl355_api_print_t print;
	custom_adxl355_api_open_t open;
	custom_adxl355_api_read_reg_t read_reg;
	custom_adxl355_api_write_reg_t write_reg;
	custom_adxl355_api_close_t close;
};

__syscall     void        custom_adxl355_print(const struct device * dev);

/* STEP 4.4 - Implement the Z_impl_* translation function to call the device driver API for this feature */
static inline void z_impl_custom_adxl355_print(const struct device * dev)
{
	const struct custom_adxl355_driver_api *api = dev->api;

	__ASSERT(api->print, "Callback pointer should not be NULL");

	api->print(dev);
}

__syscall	int  custom_adxl355_open(const struct device * dev);
static inline int z_impl_custom_adxl355_open(const struct device * dev)
{
	const struct custom_adxl355_driver_api *api = dev->api;

	__ASSERT(api->open, "Callback pointer should not be NULL");

	return api->open(dev);
}

__syscall   int  custom_adxl355_read_reg(const struct device * dev, uint8_t reg, uint8_t *data, int size);
static inline int z_impl_custom_adxl355_read_reg(const struct device * dev, uint8_t reg, uint8_t *data, int size)
{
	const struct custom_adxl355_driver_api *api = dev->api;

	__ASSERT(api->read_reg, "Callback pointer should not be NULL");

	return api->read_reg(dev, reg, data, size);
}

__syscall   int  custom_adxl355_write_reg(const struct device * dev, uint8_t reg, uint8_t value);
static inline int z_impl_custom_adxl355_write_reg(const struct device * dev, uint8_t reg, uint8_t value)
{
	const struct custom_adxl355_driver_api *api = dev->api;

	__ASSERT(api->write_reg, "Callback pointer should not be NULL");

	return api->write_reg(dev, reg, value);
}

__syscall   int  custom_adxl355_close(const struct device * dev);
static inline int z_impl_custom_adxl355_close(const struct device * dev)
{
	const struct custom_adxl355_driver_api *api = dev->api;

	__ASSERT(api->close, "Callback pointer should not be NULL");

	return api->close(dev);
}

#ifdef __cplusplus
}
#endif

#include <syscalls/custom_adxl355_driver_spi.h>

#endif /* __CUSTOM_ADXL355_DRIVER_H__ */
